@@ -32,15 +32,27 @@ fn strip_comments(mut line: String) -> String {
 // to this zone, as a string representation of a static slice.
 fn format_rest(rest: Vec<(i64, FixedTimespan)>) -> String {
     let mut ret = "&[\n".to_string();
-    for (start, FixedTimespan { utc_offset, dst_offset, name }) in rest {
+    for (
+        start,
+        FixedTimespan {
+            utc_offset,
+            dst_offset,
+            name,
+        },
+    ) in rest
+    {
+        let timespan_name = match name.as_ref() {
+            "%z" => None,
+            name => Some(name),
+        };
         ret.push_str(&format!(
             "                    ({start}, FixedTimespan {{ \
-             utc_offset: {utc}, dst_offset: {dst}, name: \"{name}\" \
+             utc_offset: {utc}, dst_offset: {dst}, name: {name:?} \
              }}),\n",
             start = start,
             utc = utc_offset,
             dst = dst_offset,
-            name = name,
+            name = timespan_name,
         ));
     }
     ret.push_str("                ]");
@@ -51,12 +63,17 @@ fn format_rest(rest: Vec<(i64, FixedTimespan)>) -> String {
 // it's a hyphen, in which case remove it. This is so the names can be used
 // as rust identifiers.
 fn convert_bad_chars(name: &str) -> String {
-    let name = name.replace("/", "__").replace("+", "Plus");
+    let name = name.replace('/', "__").replace('+', "Plus");
     if let Some(pos) = name.find('-') {
-        if name[pos + 1..].chars().next().map(char::is_numeric).unwrap_or(false) {
-            name.replace("-", "Minus")
+        if name[pos + 1..]
+            .chars()
+            .next()
+            .map(char::is_numeric)
+            .unwrap_or(false)
+        {
+            name.replace('-', "Minus")
         } else {
-            name.replace("-", "")
+            name.replace('-', "")
         }
     } else {
         name
@@ -67,8 +84,15 @@ fn convert_bad_chars(name: &str) -> String {
 // database. The `Wrap` wrapper in the `timezone_impl` module then implements
 // TimeZone for any contained struct that implements `Timespans`.
 fn write_timezone_file(timezone_file: &mut File, table: &Table) -> io::Result<()> {
-    let zones = table.zonesets.keys().chain(table.links.keys()).collect::<BTreeSet<_>>();
-    writeln!(timezone_file, "use core::fmt::{{self, Debug, Display, Formatter}};",)?;
+    let zones = table
+        .zonesets
+        .keys()
+        .chain(table.links.keys())
+        .collect::<BTreeSet<_>>();
+    writeln!(
+        timezone_file,
+        "use core::fmt::{{self, Debug, Display, Formatter}};",
+    )?;
     writeln!(timezone_file, "use core::str::FromStr;\n",)?;
     writeln!(
         timezone_file,
@@ -83,7 +107,10 @@ fn write_timezone_file(timezone_file: &mut File, table: &Table) -> io::Result<()
 /// for details."
     )?;
     writeln!(timezone_file, "#[derive(Clone, Copy, PartialEq, Eq, Hash)]")?;
-    writeln!(timezone_file, r#"#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]"#)?;
+    writeln!(
+        timezone_file,
+        r#"#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]"#
+    )?;
     writeln!(timezone_file, "pub enum Tz {{")?;
     for zone in &zones {
         let zone_name = convert_bad_chars(zone);
@@ -100,39 +127,50 @@ fn write_timezone_file(timezone_file: &mut File, table: &Table) -> io::Result<()
     for zone in &zones {
         map.entry(zone, &format!("Tz::{}", convert_bad_chars(zone)));
     }
-    writeln!(timezone_file, "static TIMEZONES: ::phf::Map<&'static str, Tz> = \n{};", map.build())?;
+    writeln!(
+        timezone_file,
+        "static TIMEZONES: ::phf::Map<&'static str, Tz> = \n{};",
+        map.build()
+    )?;
 
     #[cfg(feature = "case-insensitive")]
     {
         writeln!(timezone_file, "use uncased::UncasedStr;\n",)?;
         let mut map = phf_codegen::Map::new();
         for zone in &zones {
-            map.entry(uncased::UncasedStr::new(zone), &format!("Tz::{}", convert_bad_chars(zone)));
+            map.entry(
+                uncased::UncasedStr::new(zone),
+                &format!("Tz::{}", convert_bad_chars(zone)),
+            );
         }
         writeln!(
             timezone_file,
             "static TIMEZONES_UNCASED: ::phf::Map<&'static uncased::UncasedStr, Tz> = \n{};",
-            // FIXME(petrosagg): remove this once rust-phf/rust-phf#232 is released
-            map.build().to_string().replace("::std::mem::transmute", "::core::mem::transmute")
+            map.build()
         )?;
     }
 
     writeln!(
         timezone_file,
-        "#[cfg(feature = \"std\")]
-pub type ParseError = String;
-#[cfg(not(feature = \"std\"))]
-pub type ParseError = &'static str;
+        r#"#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub struct ParseError(());
+
+impl Display for ParseError {{
+    fn fmt(&self, f: &mut Formatter) -> fmt::Result {{
+        f.write_str("failed to parse timezone")
+    }}
+}}
+
+#[cfg(feature = "std")]
+impl std::error::Error for ParseError {{}}
 
 impl FromStr for Tz {{
     type Err = ParseError;
     fn from_str(s: &str) -> Result<Self, Self::Err> {{
-        #[cfg(feature = \"std\")]
-        return TIMEZONES.get(s).cloned().ok_or_else(|| format!(\"'{{}}' is not a valid timezone\", s));
-        #[cfg(not(feature = \"std\"))]
-        return TIMEZONES.get(s).cloned().ok_or(\"received invalid timezone\");
+        return TIMEZONES.get(s).cloned().ok_or(ParseError(()));
     }}
-}}\n"
+}}
+"#
     )?;
 
     writeln!(
@@ -164,10 +202,7 @@ impl FromStr for Tz {{
     #[cfg(feature = "case-insensitive")]
     /// Parses a timezone string in a case-insensitive way
     pub fn from_str_insensitive(s: &str) -> Result<Self, ParseError> {{
-        #[cfg(feature = "std")]
-        return TIMEZONES_UNCASED.get(s.into()).cloned().ok_or_else(|| format!("'{{}}' is not a valid timezone", s));
-        #[cfg(not(feature = "std"))]
-        return TIMEZONES_UNCASED.get(s.into()).cloned().ok_or("received invalid timezone");
+        return TIMEZONES_UNCASED.get(s.into()).cloned().ok_or(ParseError(()));
     }}"#
         )?;
     }
@@ -197,8 +232,12 @@ impl FromStr for Tz {{
         match *self {{"
     )?;
     for zone in &zones {
-        let timespans = table.timespans(&zone).unwrap();
+        let timespans = table.timespans(zone).unwrap();
         let zone_name = convert_bad_chars(zone);
+        let timespan_name = match timespans.first.name.as_ref() {
+            "%z" => None,
+            name => Some(name),
+        };
         writeln!(
             timezone_file,
             "            Tz::{zone} => {{
@@ -207,7 +246,7 @@ impl FromStr for Tz {{
                     first: FixedTimespan {{
                         utc_offset: {utc},
                         dst_offset: {dst},
-                        name: \"{name}\",
+                        name: {name:?},
                     }},
                     rest: REST
                 }}
@@ -216,7 +255,7 @@ impl FromStr for Tz {{
             rest = format_rest(timespans.rest),
             utc = timespans.first.utc_offset,
             dst = timespans.first.dst_offset,
-            name = timespans.first.name,
+            name = timespan_name,
         )?;
     }
     write!(
@@ -240,7 +279,11 @@ pub static TZ_VARIANTS: [Tz; {num}] = [
         num = zones.len()
     )?;
     for zone in &zones {
-        writeln!(timezone_file, "    Tz::{zone},", zone = convert_bad_chars(zone))?;
+        writeln!(
+            timezone_file,
+            "    Tz::{zone},",
+            zone = convert_bad_chars(zone)
+        )?;
     }
     write!(timezone_file, "];")?;
     Ok(())
@@ -248,8 +291,13 @@ pub static TZ_VARIANTS: [Tz; {num}] = [
 
 // Create a file containing nice-looking re-exports such as Europe::London
 // instead of having to use chrono_tz::timezones::Europe__London
-fn write_directory_file(directory_file: &mut File, table: &Table) -> io::Result<()> {
-    // add the `loose' zone definitions first at the top of the file
+fn write_directory_file(directory_file: &mut File, table: &Table, version: &str) -> io::Result<()> {
+    // expose the underlying IANA TZDB version
+    writeln!(
+        directory_file,
+        "pub const IANA_TZDB_VERSION : &str = \"{version}\";\n"
+    )?;
+    // add the `loose' zone definitions first
     writeln!(directory_file, "use crate::timezones::Tz;\n")?;
     let zones = table
         .zonesets
@@ -259,7 +307,11 @@ fn write_directory_file(directory_file: &mut File, table: &Table) -> io::Result<
         .collect::<BTreeSet<_>>();
     for zone in zones {
         let zone = convert_bad_chars(zone);
-        writeln!(directory_file, "pub const {name} : Tz = Tz::{name};", name = zone)?;
+        writeln!(
+            directory_file,
+            "pub const {name} : Tz = Tz::{name};",
+            name = zone
+        )?;
     }
     writeln!(directory_file)?;
 
@@ -275,7 +327,11 @@ fn write_directory_file(directory_file: &mut File, table: &Table) -> io::Result<
             match child {
                 Child::Submodule(name) => {
                     let submodule_name = convert_bad_chars(name);
-                    writeln!(directory_file, "    pub mod {name} {{", name = submodule_name)?;
+                    writeln!(
+                        directory_file,
+                        "    pub mod {name} {{",
+                        name = submodule_name
+                    )?;
                     writeln!(directory_file, "        use crate::timezones::Tz;\n",)?;
                     let full_name = entry.name.to_string() + "/" + name;
                     for entry in table.structure() {
@@ -324,6 +380,8 @@ mod filter {
 }
 
 /// Module containing code supporting filter-by-regex feature
+///
+/// The "GMT" and "UTC" time zones are always included.
 #[cfg(feature = "filter-by-regex")]
 mod filter {
     use std::collections::HashSet;
@@ -386,13 +444,13 @@ mod filter {
     fn filter_timezone_table(table: &mut Table, filter_regex: Regex) {
         // Compute the transitive closure of things to keep.
         // Doing this, instead of just filtering `zonesets` and `links` by the
-        // regiex, helps to keep the `structure()` intact.
+        // regex, helps to keep the `structure()` intact.
         let mut keep = HashSet::new();
         for (k, v) in &table.links {
-            if filter_regex.is_match(k) {
+            if filter_regex.is_match(k) || k == "GMT" || k == "UTC" {
                 insert_keep_entry(&mut keep, k);
             }
-            if filter_regex.is_match(v) {
+            if filter_regex.is_match(v) || k == "GMT" || k == "UTC" {
                 insert_keep_entry(&mut keep, v);
             }
         }
@@ -422,14 +480,37 @@ mod filter {
         }
 
         // Actually do the filtering.
-        table.links.retain(|k, v| keep.contains(k) || keep.contains(v));
+        table
+            .links
+            .retain(|k, v| keep.contains(k) || keep.contains(v));
 
         table
             .zonesets
-            .retain(|k, _| filter_regex.is_match(&k) || keep.iter().any(|s| k.starts_with(s)));
+            .retain(|k, _| filter_regex.is_match(k) || keep.iter().any(|s| k.starts_with(s)));
     }
 }
 
+fn detect_iana_db_version() -> String {
+    let root = env::var("CARGO_MANIFEST_DIR").expect("no Cargo build context");
+    let path = Path::new(&root).join(Path::new("tz/NEWS"));
+    let file = File::open(path).expect("failed to open file");
+
+    let mut lines = BufReader::new(file).lines();
+    while let Some(Ok(line)) = lines.next() {
+        let line = match line.strip_prefix("Release ") {
+            Some(line) => line,
+            _ => continue,
+        };
+
+        match line.split_once(" - ") {
+            Some((version, _)) => return version.to_owned(),
+            _ => continue,
+        }
+    }
+
+    unreachable!("no version found")
+}
+
 pub fn main() {
     println!("cargo:rerun-if-env-changed={}", FILTER_ENV_VAR_NAME);
 
@@ -476,10 +557,11 @@ pub fn main() {
     filter::maybe_filter_timezone_table(&mut table);
 
     let timezone_path = Path::new(&env::var("OUT_DIR").unwrap()).join("timezones.rs");
-    let mut timezone_file = File::create(&timezone_path).unwrap();
+    let mut timezone_file = File::create(timezone_path).unwrap();
     write_timezone_file(&mut timezone_file, &table).unwrap();
 
     let directory_path = Path::new(&env::var("OUT_DIR").unwrap()).join("directory.rs");
-    let mut directory_file = File::create(&directory_path).unwrap();
-    write_directory_file(&mut directory_file, &table).unwrap();
+    let mut directory_file = File::create(directory_path).unwrap();
+    let version = detect_iana_db_version();
+    write_directory_file(&mut directory_file, &table, &version).unwrap();
 }
